/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "printproject.h"
#include "utilities.h"
#include "bible.h"
#include "book.h"
#include "chapter.h"
#include "usfmtools.h"
#include "log.h"
#include <config.h>
#include "scripture.h"
#include "pdfviewer.h"
#include "xmlutils.h"
#include "paper.h"
#include "xmlfo-utils.h"
#include "project.h"
#include "style.h"
#include "stylesheetutils.h"
#include "fop.h"
#include "constants.h"


void view_project_pdf (Configuration * configuration, Usfm * usfm,
                       GtkWidget * progressbar, GtkWidget * notebook)
// Formats the project and shows it in a pdf viewer.
{
  // Log.
  log_message ("Starting to print the project");
  
  // Initialize progressbar.
  double percentage = 0;
  string progresstext ("Starting");
  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar), percentage);
  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar), progresstext.c_str());
  // Notebook shows progress bar.
  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 1);
  // Update display.
  while (gtk_events_pending ()) gtk_main_iteration ();

  // Storage for xsl-fo file.
  vector<ustring> xslfo_lines;

  // Scripture related data.
  Scripture scripture (configuration->project);
  vector<ustring> running_headers;

  // Data for progress bar.
  double progresscounter = 0;
  double progressmaximum = 2.25 * scripture.books.size();
  
  // Start building the xml-fo file.
  // The various elements of this file are created using objects.
  // When the object goes out of scope, it writes the closing elements.
  {
    XmlFoRoot xmlforoot (&xslfo_lines, "serif");
    {
      XmlFoLayoutMasterSet layoutmasterset (&xslfo_lines, configuration, true);
    }
    {
      XmlFoPageSequence pagesequence (&xslfo_lines);
      // Generate the table of contents.
      {
        XmlFoFlow flow (&xslfo_lines);
        {
          XmlFoBlock block (&xslfo_lines, 18, false, true, "", 0, 5, 0, 0, 0, true);
          xslfo_lines.push_back("Table of Contents");
        }
        // Go through all the books and collect the running header for each book.
        for (unsigned int i = 0; i < scripture.books.size(); i++) {
          // Update progress bar.
          progresscounter += 1;
          percentage = progresscounter / progressmaximum;
          progresstext = "Table of contents";
          gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar), percentage);
          gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar), progresstext.c_str());
          while (gtk_events_pending ()) gtk_main_iteration ();
          // Get the running header.
          string booktitle;
          Book book (scripture.paths[i], false, false, usfm);
          // Get the text of the first chapter.
          vector <ustring> lines;
          book.get_chapter (1, lines);
          // Search for data in the first couple of lines.
          unsigned int high_limit;
          high_limit = MIN (10, lines.size());
          for (unsigned int i2 = 0; i2 < high_limit; i2++) {
            ustring line = lines[i2];
            ustring marker = usfm_extract_marker (line);
            if (usfm->is_running_header (marker))
              booktitle = line;
          }      
          // No running header found? Take English name.
          if (booktitle.empty()) 
            booktitle = scripture.books[i];
          // Store for later use.
          running_headers.push_back (booktitle);
          // Write the xsl-fo data.
          XmlFoBlock block (&xslfo_lines, 12, false, false, "last-justify", 0, 0, 0, 0, 0, true);
          xslfo_lines.push_back(booktitle);
          xslfo_lines.push_back("        <fo:leader leader-pattern=\"dots\"/>");
          ustring line;
          line = "        <fo:page-number-citation ref-id=\"";
          line.append (booktitle);
          line.append ("\"/>");
          xslfo_lines.push_back(line);
        }
      }
    }

    // Generate all the books.
    for (unsigned int i = 0; i < scripture.books.size(); i++) {
      // Update progress bar.
      progresscounter += 1;
      percentage = progresscounter / progressmaximum;
      progresstext = running_headers[i];
      gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar), percentage);
      gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar), progresstext.c_str());
      while (gtk_events_pending ()) gtk_main_iteration ();
      // Make page sequence, static content, and flow for the book.        
      XmlFoPageSequence pagesequence (&xslfo_lines);
      {
        XmlFoStaticContent staticcontent (&xslfo_lines, configuration);
      }
      {
        XmlFoFlow flow (&xslfo_lines);
        // Generate markers for the table of contents.
        ustring line;
        line = "      <fo:block id=\"toc-id\" span=\"all\">";
        // (The span="all" is to work around a bug in FOP).
        replace_text (line, "toc-id", running_headers[i]);
        xslfo_lines.push_back (line);
        xslfo_lines.push_back ("        <fo:marker marker-class-name=\"book\">");
        xslfo_lines.push_back(running_headers[i]);
        xslfo_lines.push_back("        </fo:marker>");
        xslfo_lines.push_back("      </fo:block>");
        // Open the book.
        Book book (scripture.paths[i], false, false, usfm);
        // Go through all the lines of the book.
        XmlFoBlock * xmlfoblock = NULL;
        for (unsigned int i2 = 0; i2 < book.cleaned_lines.size(); i2++) {
          ustring line = book.cleaned_lines[i2];
          // Change certain characters to xml entities.
          vector <size_t> dummy;
          xml_handle_entities (line, dummy);
          // Deal with possible "inserted" and "removed" marks.
          replace_text (line, STRIKE_THROUGH_BEGIN, "<fo:inline text-decoration=\"line-through\">");
          replace_text (line, STRIKE_THROUGH_END, "</fo:inline>");
          replace_text (line, BOLD_BEGIN, "<fo:inline font-weight=\"bold\">");
          replace_text (line, BOLD_END, "</fo:inline>");
          // Get the style belonging to the marker.
          ustring marker = usfm_extract_marker (line);
          if (usfm->is_identifier (marker)) {
            // Do nothing with an identifier.
          } else if (usfm->is_comment (marker)) {
            // Skip comments.
          } else if (usfm->is_running_header (marker)) {
            // Skip running header, because it has been dealth with above.
          } else if (usfm->is_starting_paragraph (marker)) {
            if (xmlfoblock) {
              delete xmlfoblock;
              xmlfoblock = NULL;
            }
            xmlfoblock = new XmlFoBlock (&xslfo_lines, 
              usfm->fontsize(), usfm->italics(), usfm->bold(), usfm->justification(),
              usfm->spacebefore(), usfm->spaceafter(), usfm->leftmargin(),
              usfm->rightmargin(), usfm->firstlineindent(), usfm->spancolumns());
            xslfo_lines.push_back(line);
          } else if (usfm->is_inline_text (marker)) {
            // Inline text, deal with it: add formatting.
            xslfo_lines.push_back(XmlFoInlineText (line, usfm->fontsize(), usfm->italics(), usfm->bold(), usfm->superscript()));
          } else if (usfm->is_chapter_number (marker)) {
            // Close any open block.
            if (xmlfoblock) {
              delete xmlfoblock;
              xmlfoblock = NULL;
            }
            // Insert marker for chapter number.
            xslfo_lines.push_back ("      <fo:block>");
            xslfo_lines.push_back ("        <fo:marker marker-class-name=\"chapter\">");
            xslfo_lines.push_back (line);
            xslfo_lines.push_back ("        </fo:marker>");
            xslfo_lines.push_back ("      </fo:block>");
            // Insert chapter text.
            xmlfoblock = new XmlFoBlock (&xslfo_lines, 
              usfm->fontsize(), usfm->italics(), usfm->bold(), usfm->justification(),
              usfm->spacebefore(), usfm->spaceafter(), usfm->leftmargin(),
              usfm->rightmargin(), usfm->firstlineindent(), usfm->spancolumns());
            xslfo_lines.push_back(line);
          } else if (usfm->is_verse_number (marker)) {
            // When the usfm is a verse number, then the number is put in the 
            // format specified by the stylesheet, but the remaining part of the
            // line inherits the formatting from the block it is in, usually
            // a paragraph block.
            ustring versenumber = number_in_string (line);
            xslfo_lines.push_back(XmlFoInlineText (versenumber, usfm->fontsize(), usfm->italics(), usfm->bold(), usfm->superscript()));
            line.erase (0, versenumber.length());
            line = trim (line);
            xslfo_lines.push_back(line);
          } else {
            // Fallback for unknown marker.
            xslfo_lines.push_back(line);
          }
        } 
        // Close possible last block of text.
        if (xmlfoblock) {
          delete xmlfoblock;
        }
        // Balance text columns using a block that spans the columns.
        xslfo_lines.push_back ("<fo:block span=\"all\"></fo:block>");
      }
    }
  }    
  
  // Make a temporary directory where to put the working files and the resulting
  // .pdf file.information. This directory is not removed, because the pdf viewer 
  // needs the .pdf file to be there during viewing or printing.
  string working_directory = g_build_filename(g_get_tmp_dir(), "bibledit_project", NULL);
  create_directory (working_directory);
  // Produce filename of .fo file.
  string fofilename = g_build_filename(working_directory.c_str(), "document.fo", NULL);
  // Write the document.
  write_lines (fofilename, xslfo_lines);
  // Tell user to be patient while we pdfxmltex runs.  
  progresstext = "Typesetting pages ...";
  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar), progresstext.c_str());
  while (gtk_events_pending ()) gtk_main_iteration ();
  // Convert the xsl-fo document to .pdf.
  string pdffilename = g_build_filename(working_directory.c_str(), "document.pdf", NULL);
  int result = fop_convert_to_pdf (configuration, fofilename, pdffilename);
  if (result != 0) {
    GtkWidget *dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "There was a problem in FOP.");
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
  }
  // Progressbar: ready.
  percentage = 1;
  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar), percentage);
  progresstext = "Ready";
  gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar), progresstext.c_str());
  while (gtk_events_pending ()) gtk_main_iteration ();
  // View the .pdf document.
  if (result == 0)
    pdfviewer (pdffilename);
  // Finalize progress bar.
  percentage = 0;
  gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar), percentage);
  gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 0);
  while (gtk_events_pending ()) gtk_main_iteration ();

  // Log: ready.
  log_message ("Ready printing the project");
}
