/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include "stylesheetutils.h"
#include "directories.h"
#include <config.h>
#include "constants.h"
#include <set>


void stylesheet_get_ones_available (vector<string>& names)
// Gets the names of the stylesheets that are there.
{
  ReadDirectories rd (directories_get_stylesheets (), "", "");
  names.assign (rd.directories.begin(), rd.directories.end());
}


void stylesheet_create_new (const string& name)
// Create a new stylesheet from templates, named "name".
{
  string directory;
  directory = directories_get_stylesheets ();
  directory = g_build_filename (directory.c_str(), name.c_str(), NULL);
  create_directory (directory);
  ReadFiles rf (PACKAGE_DATA_DIR, "", ".style");
  for (unsigned int i = 0; i < rf.files.size(); i++) {
    string command;
    command = "cp ";
    string file = g_build_filename (PACKAGE_DATA_DIR, rf.files[i].c_str(), NULL);
    command.append (file);
    command.append (" '");
    file = rf.files[i];
    file = file.substr (0, file.length() - 6);
    file = g_build_filename (directory.c_str(), file.c_str(), NULL);
    command.append (file);
    command.append ("'");
    system(command.c_str());
  }
}


void stylesheet_delete (const string& name)
// Deletes a stylesheet.
{
  string directory;
  directory = directories_get_stylesheets ();
  directory = g_build_filename (directory.c_str(), name.c_str(), NULL);
  string command;
  command = "rm -rf '";
  command.append (directory);
  command.append ("'");
  system(command.c_str());
}


void stylesheet_copy (const string& from_name, const string& to_name)
// Copies one stylesheet to another.
{
  string from_dir;
  from_dir = directories_get_stylesheets ();
  from_dir = g_build_filename (from_dir.c_str(), from_name.c_str(), NULL);
  string to_dir;
  to_dir = directories_get_stylesheets ();
  to_dir = g_build_filename (to_dir.c_str(), to_name.c_str(), NULL);
  string command;
  command = "cp -r '";
  command.append (from_dir);
  command.append ("' '");
  command.append (to_dir);
  command.append ("'");
  system (command.c_str());
}


void stylesheet_get_styles (const string& stylesheet, vector<Style>& styles)
// Get all data of all stylesheets.
{
  // When a project uses a stylesheet that does not exist, use another one,
  // or create one.
  string checked_stylesheet (stylesheet);
  vector<string> stylesheets;
  stylesheet_get_ones_available (stylesheets);
  if (stylesheets.size() == 0) {
    // No stylesheets: create "Standard".
    stylesheet_create_new (STANDARDSHEET);
    stylesheets.push_back (STANDARDSHEET);
  }
  set<string> sheets (stylesheets.begin(), stylesheets.end());
  if (sheets.find (checked_stylesheet) != sheets.end())
    checked_stylesheet = checked_stylesheet;
  else if (sheets.find (STANDARDSHEET) != sheets.end())
    checked_stylesheet = STANDARDSHEET;
  else
    checked_stylesheet = stylesheets[0];
  // We definitely have an existing stylesheet now.  
  string directory;
  directory = directories_get_stylesheets ();
  directory = g_build_filename (directory.c_str(), checked_stylesheet.c_str(), NULL);
  ReadFiles rf (directory, "", "");
  for (unsigned int i = 0; i < rf.files.size(); i++) {
    Style style (directory, rf.files[i], false);
    styles.push_back(style);
  }
}


void stylesheet_delete_style (const string& stylesheet, const string& marker)
// Deletes a style.
{
  string filename;
  filename = directories_get_stylesheets ();
  filename = g_build_filename (filename.c_str(), stylesheet.c_str(), marker.c_str(), NULL);
  unlink (filename.c_str());  
}


void stylesheet_new_style (const string& stylesheet, const string& marker, const string& name, const string& info)
// Adds a new style.
{
  string directory;
  directory = directories_get_stylesheets ();
  directory = g_build_filename (directory.c_str(), stylesheet.c_str(), NULL);
  Style style (directory, marker, true);
  style.name = name;
  style.info = info;
}


void stylesheet_save_style (const string& stylesheet, const string& marker,
                            StyleType type, double fontsize, 
                            bool italics, bool bold, bool superscript,
                            string justification,
                            double spacebefore, double spaceafter,
                            double leftmargin, double rightmargin,
                            double firstlineindent, bool spancolumns)
// Saves style information.
{
  string directory;
  directory = directories_get_stylesheets ();
  directory = g_build_filename (directory.c_str(), stylesheet.c_str(), NULL);
  Style style (directory, marker, true);
  style.type = type;
  style.fontsize = fontsize;
  style.italics = italics;
  style.bold = bold;
  style.superscript = superscript;
  style.justification = justification;
  style.spacebefore = spacebefore;
  style.spaceafter = spaceafter;
  style.leftmargin = leftmargin;
  style.rightmargin = rightmargin;
  style.firstlineindent = firstlineindent;
  style.spancolumns = spancolumns;
}


unsigned int stylesheet_style_get_pointer (const vector<Style>& styles, const string& marker)
// Returns a pointer to "styles" which describes "marker".
// Fallback on marker "p".
{
  // Search marker and return pointer.
  for (unsigned int i = 0; i < styles.size(); i++)
    if (marker == styles[i].marker)
      return i;
  // Ok, marker not found, return pointer to \p
  for (unsigned int i = 0; i < styles.size(); i++)
    if (styles[i].marker == "p")
      return i;
  // Ok, even \p not found, return 0.
  return 0;
}
