/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "usfm.h"
#include "utilities.h"
#include "config.h"
#include "project.h"
#include "directories.h"
#include "style.h"
#include "stylesheetutils.h"


Usfm::Usfm (const string& stylesheet)
{
  stylesheet_get_styles (stylesheet, styles);
}


Usfm::~Usfm ()
{
}


bool Usfm::is_identifier (const string& usfm)
{
  get_style_pointer (usfm);
  return (styles[last_style_pointer].type == stIdentifier);
}


bool Usfm::is_comment (const string& usfm)
{
  get_style_pointer (usfm);
  return (styles[last_style_pointer].type == stComment);
}


bool Usfm::is_running_header (const string& usfm)
{
  get_style_pointer (usfm);
  return (styles[last_style_pointer].type == stRunningHeader);
}


bool Usfm::is_starting_paragraph (const string& usfm)
{
  get_style_pointer (usfm);
  return (styles[last_style_pointer].type == stStartsParagraph);
}


bool Usfm::is_inline_text (const string& usfm)
{
  get_style_pointer (usfm);
  return (styles[last_style_pointer].type == stInlineText);
}


bool Usfm::is_chapter_number (const string& usfm)
{
  get_style_pointer (usfm);
  return (styles[last_style_pointer].type == stChapterNumber);
}


bool Usfm::is_verse_number (const string& usfm)
{
  get_style_pointer (usfm);
  return (styles[last_style_pointer].type == stVerseNumber);
}


double Usfm::fontsize ()
{
  return styles[last_style_pointer].fontsize;
}


bool Usfm::italics ()
{
  return styles[last_style_pointer].italics;
}


bool Usfm::bold ()
{
  return styles[last_style_pointer].bold;
}


bool Usfm::superscript ()
{
  return styles[last_style_pointer].superscript;
}


string Usfm::justification ()
{
  return styles[last_style_pointer].justification;
}


double Usfm::spacebefore ()
{
  return styles[last_style_pointer].spacebefore;
}


double Usfm::spaceafter ()
{
  return styles[last_style_pointer].spaceafter;
}


double Usfm::leftmargin ()
{
  return styles[last_style_pointer].leftmargin;
}


double Usfm::rightmargin ()
{
  return styles[last_style_pointer].rightmargin;
}


double Usfm::firstlineindent ()
{
  return styles[last_style_pointer].firstlineindent;
}


bool Usfm::spancolumns ()
{
  return styles[last_style_pointer].spancolumns;
}


unsigned int Usfm::get_style_pointer (const string& usfm)
// Returns a pointer to the style relevant for "usfm".
{
  // Optimization to speed up finding the pointer: if found before, reuse the result.
  if (usfm != last_marker) {
    last_style_pointer = stylesheet_style_get_pointer (styles, usfm);
  }
  return last_style_pointer;
}


#define USFM_MAX_DATA 65
char * USFM_DATA[USFM_MAX_DATA] = {
  /*
  Here we've data related to the Unified Standard Format Markers.
  Column 1: the marker.
  Column 2: the name of the marker.
  Column 3: elaborate information on the marker.
  Column 4: whether the marker should be inserted on a new line.
  Column 5: whether a space should be inserted after the marker.
  */
  // Identification information
  "id", "Identification line", "Identifies the file.", "1", "1",
  "h", "Running header", "The book title heading to be included at the top of each printed page. Entered only once at the beginning of the file following \\id.", "1", "1",
  // Title headings and lavels
  "mt", "Major title", "The key component in the title of e.g. a biblical book.", "1", "1",
  "mt1", "Major title, level 1", "The key component in the title of e.g. biblical book.", "1", "1",
  "mt2", "Major title, level 2", "The key component in the title of e.g. biblical book, but with the lesser emphasis.", "1", "1",
  "s", "Section heading", "The standard and common section heading.", "1", "1",
  // Chapters and verses
  "c", "Chapter number", "The marker is followed by the chapter number. A space is in between. No text should follow this marker.", "1", "1",
  "v", "Verse number", "The marker is followed by the verse number. A space is in between. Then follows the text of the verse.", "1", "1",
  // Paragraph elements
  "p", "Normal paragraph", "Followed immediately by a space and paragraph text, or by a new line and a verse marker.", "1", "0",
  // Peripheral materials
  "id FRT", "Front matter identification", "Identification line for the front matter.", "1", "0",
  "id BAK", "Back matter identification", "Identification line for the back matter.", "1", "0",
  "id OTH", "Other material identification", "Identification line for other pheripheral material.", "1", "0",
  "cov", "Cover", "Sub-group marker for cover", "1", "0"
};


#define USFM_MAX_DATA2 11
char * KNOWN_USFMS[USFM_MAX_DATA2] = {
  // Identification information
  "id",
  "h",
  // Title headings and lavels
  "mt",
  "mt1",
  "mt2",
  "s",
  // Chapters and verses
  "c",
  "v",
  // Paragraph elements
  "p",
  "nb",
  // Peripheral materials
  "cov"
};


bool INSERT_ON_NEWLINE[USFM_MAX_DATA2] = {
  true, // id
  true, // h
  true, // mt
  true, // mt1
  true, // mt2
  true, // s
  true, // c
  true, // v
  true, // p
  true, // nb
  true // cov
};


bool INSERT_SPACE_AFTER[USFM_MAX_DATA2] = {
  true, // id
  true, // h
  true, // mt
  true, // mt1
  true, // mt2
  true, // s
  true, // c
  true, // v
  false, // p
  false, // nb
  false // cov
};


string usfm_get_full_name (const string& usfm)
{
  string fullname = "Unknown marker";
  for (unsigned int i = 0; i < USFM_MAX_DATA; i += 5) {
    if (usfm == USFM_DATA[i]) {
      fullname = USFM_DATA[i + 1];
      break;
    }
  }
  return fullname;
  // Todo update using new data.
}


string usfm_get_marker (const string& fullname)
{
  string marker;
  for (unsigned int i = 1; i < USFM_MAX_DATA; i += 5) {
    if (fullname == USFM_DATA[i]) {
      marker = USFM_DATA[i - 1];
      break;
    }
  }
  return marker;
  // Todo update using new data.
}


string usfm_get_comment (const string& usfm)
{
  string comment = "Unknown marker";
  for (unsigned int i = 0; i < USFM_MAX_DATA; i += 5) {
    if (usfm == USFM_DATA[i]) {
      comment = USFM_DATA[i + 2];
      break;
    }
  }
  return comment;
  // Todo update using new data.
}


bool usfm_is_fixed (const string& usfm)
// Returns whether this usfm is fixed, that is,
// it is known and supported in the program code.
{
  for (unsigned int i = 0; i < USFM_MAX_DATA2; i++) {
    if (usfm == KNOWN_USFMS[i]) {
      return true;
    }
  }
  return false;
}


bool usfm_starts_newline (const string& usfm)
// Indicates whether an usfm, when inserted in the text, should start a new line.
{
  for (unsigned int i = 0; i < USFM_MAX_DATA; i += 5) {
    if (usfm == USFM_DATA[i]) {
      if (USFM_DATA[i + 3] == "1") return true;
      else return false;
    }
  }
  return true;
  // Todo update using new data.
}


bool usfm_insert_space_after (const string& usfm)
// Whether the editor should insert a space after the usfm.
{
  for (unsigned int i = 0; i < USFM_MAX_DATA; i += 5) {
    if (usfm == USFM_DATA[i]) {
      if (USFM_DATA[i + 4] == "1") return true;
      else return false;
    }
  }
  return false;
  // Todo update using new data.
}
